/*  ┌──────────────────────────── ♫
    ║Programmer: Jonathan Cunti
    ║Date: 2022/01/05
    ├──────────────────────────── ♫
    ║Program: Sad Cat Fixer
    ├──────────────────────────── ♫
    ║Description: using an ultra sonic sensor to move
    ║a servo motor with an attached ball for the cat
    ║to chase
    ├──────────────────────────── ♫
    ║Connections:
    ║ GND, 5V, 5, 6, 2, 11
    └──────────────────────────── ♫
*/

//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
//Global Section: contains libraries and variables
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

#include <Servo.h> //inputs the servo library into the program 
#include <IRremote.h> //inputs the IRemote library into the program 
Servo myservo; //creates a my servo object used to control the servo motor
int Servogoto; //creates a variable to store the value that the motor will turn to

int RECV_PIN = 11; //assignes the variable RECV_PIN to pin 11
IRrecv irrecv(RECV_PIN); //tells the program the RECV_PIN will be reading the ultrasonic waves
decode_results results; //variable used to store the result from the remote

int sendPin = 5; // Trigger Pin of Ultrasonic Sensor assigned to pin 5
int receivePin = 6; // Echo Pin of Ultrasonic Sensor assigned to pin 5
long cm; //creates a variable cm to store the conversion from microseconds
long duration; //stores the microseconds reading from the sensor

int mode = 1; //mode variable used to switch between the modes of the device
int buttonPin = 2; //assigned the button to pin 2

//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
//Setup Section: runs codde to setup the program and prepair for the loop
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

void setup()
{
  myservo.attach(9);//attachs the servo on pin 9 to servo object
  myservo.write(90);//moves the motor to 90 degree's
  pinMode(buttonPin, INPUT); //sets the button to an input
  delay(1000); //delay of 1 second
  Serial.begin(9600); //sets the speed at wich the program sends data to the serial monitor
  pinMode(sendPin, OUTPUT); //sets the send pin from the ultrasonic sensor as an output
  pinMode(receivePin, INPUT); //sets the receive pin from the ultrasonic sensor as as input
  irrecv.enableIRIn(); //enables the IR input to be detected
}

//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
//Loop Section: runs this loop forever
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

void loop()
{
  while (mode == 1) //while the mode is set to 1(manual control)
  {
    if (irrecv.decode(&results)) //if the remote recieves a value it will decode it
    {
      irrecv.resume(); //allows the reciever pin to recieve new information
      switch (results.value)//telling the switch to read to results.value variable
      {
        case 0xFF10EF: //if the 4 key is pressed on the remote
          myservo.write(180); //moves the servo motor to 180 degree's
          break ;

        case 0xFF30CF: //if the 1 key is pressed on the remote
          myservo.write(135); //moves the servo motor to 135 degree's
          break ;

        case 0xFF18E7: //if the 2 key is pesssed on the remote
          myservo.write(90);//moves the servo motor to 90 degree's
          break ;

        case 0xFF7A85: //if the 3 key is pressed on the remote
          myservo.write(45);//moves the servo motor to 45 degree's
          break ;

        case 0xFF5AA5: //if the 6 key is pressed on the remote
          myservo.write(0);//moves the servo motor to 0 degree's
          break ;

        case 0xFFA25D: //if the power key is pressed on the remote
          mode = 0;//switched the input to the sensor
          break ;
      }
      irrecv.resume(); //resets the receiver after if has been pinged and prepair's it to recieve more information
    }
  }
  while (mode == 0)
  {
    digitalWrite(sendPin, LOW); //turns the send pin off
    delayMicroseconds(2); //delay to keep it off for 2 ms
    digitalWrite(sendPin, HIGH); //turns the send pin on
    delayMicroseconds(10); //delay to send waves for 10 ms
    digitalWrite(sendPin, LOW); //turns the send pin off
    duration = pulseIn(receivePin, HIGH); //turns the receive pin on to see read the waves returning
    cm = converter(duration); //calls the converter function to convert the reading from duration to cm
    if (cm < 40)//checks if the cm reading is less than 40 cm
    {
      Servogoto = random(0, 180);//selects a number from 0 to 180
      myservo.write(Servogoto);//moves the servo to the degree selected by the random function
      delay(500); //delay of 500 ms to allow the servo motor to move
      if (digitalRead(buttonPin) == HIGH)//checks if the button is pressed to turn off automatic
      {
        mode = 1;//sets the mode to manual
      }
    }
    else
    {
      if (digitalRead(buttonPin) == HIGH)//checks if the button is pressed to turn off automatic
      {
        mode = 1;//sets the mode to manual
      }
    }
    printSensor(); //calls the print sensor function
  }
}

long converter(long microseconds) //converter function used to convert duration in ms to cm
{
  return microseconds / 29 / 2; //conversiton ratio to convert ms to cm
}

void printSensor()//print sensor function used to print the readings from the sensor
{
  Serial.print("Sensor: "); //prints the word sensor
  Serial.print(cm); //prints the cm reading
  Serial.print("cm"); //prints the word cm
  Serial.println("");//moves to the next line
}
